@@ -1,22 +1,28 @@
 import time
+import uuid
 from contextlib import contextmanager
 from dataclasses import dataclass
-from typing import Any, Optional, Tuple
+from typing import Any, Dict, Optional, Tuple
 
 import agate
+import clickhouse_connect
 import dbt.exceptions
-from clickhouse_driver import Client, errors
+from clickhouse_connect.driver.exceptions import DatabaseError, Error
 from dbt.adapters.base import Credentials
 from dbt.adapters.sql import SQLConnectionManager
 from dbt.contracts.connection import Connection
 from dbt.events import AdapterLogger
-from dbt.version import __version__ as dbt_version
 
 logger = AdapterLogger('clickhouse')
 
 
 @dataclass
 class ClickhouseCredentials(Credentials):
+    """
+    ClickHouse connection credentials data class.
+    """
+
+    # pylint: disable=too-many-instance-attributes
     host: str = 'localhost'
     port: Optional[int] = None
     user: Optional[str] = 'default'
@@ -25,12 +31,16 @@ class ClickhouseCredentials(Credentials):
     password: str = ''
     cluster: Optional[str] = None
     secure: bool = False
-    verify: bool = False
+    verify: bool = True
     connect_timeout: int = 10
     send_receive_timeout: int = 300
     sync_request_timeout: int = 5
     compress_block_size: int = 1048576
     compression: str = ''
+    use_default_schema: bool = (
+        False  # This is used in tests to make sure we connect always to the default database.
+    )
+    custom_settings: Optional[Dict[str, Any]] = None
 
     @property
     def type(self):
@@ -52,37 +62,38 @@ class ClickhouseCredentials(Credentials):
         self.database = None
 
     def _connection_keys(self):
-        return ('host', 'port', 'user', 'schema', 'secure', 'verify')
+        return 'host', 'port', 'user', 'schema', 'secure', 'verify'
 
 
 class ClickhouseConnectionManager(SQLConnectionManager):
+    """
+    ClickHouse Connector connection manager.
+    """
+
     TYPE = 'clickhouse'
 
     @contextmanager
     def exception_handler(self, sql):
         try:
             yield
-
-        except errors.ServerException as e:
-            logger.debug('Clickhouse error: {}', str(e))
-
+        except DatabaseError as err:
+            logger.debug('Clickhouse error: {}', str(err))
             try:
                 # attempt to release the connection
                 self.release()
-            except errors.Error:
+            except Error:
                 logger.debug('Failed to release connection!')
-                pass
 
-            raise dbt.exceptions.DatabaseException(str(e).strip()) from e
+            raise dbt.exceptions.DatabaseException(str(err).strip()) from err
 
-        except Exception as e:
+        except Exception as exp:
             logger.debug('Error running SQL: {}', sql)
             logger.debug('Rolling back transaction.')
             self.release()
-            if isinstance(e, dbt.exceptions.RuntimeException):
+            if isinstance(exp, dbt.exceptions.RuntimeException):
                 raise
 
-            raise dbt.exceptions.RuntimeException(e) from e
+            raise dbt.exceptions.RuntimeException(exp) from exp
 
     @classmethod
     def open(cls, connection):
@@ -91,37 +102,38 @@ class ClickhouseConnectionManager(SQLConnectionManager):
             return connection
 
         credentials = cls.get_credentials(connection.credentials)
-        kwargs = {}
 
         try:
-            handle = Client(
+            custom_settings = (
+                dict() if credentials.custom_settings is None else credentials.custom_settings
+            )
+            handle = clickhouse_connect.get_client(
                 host=credentials.host,
                 port=credentials.port,
-                database='default',
-                user=credentials.user,
+                database='default' if credentials.use_default_schema else credentials.schema,
+                username=credentials.user,
                 password=credentials.password,
-                client_name=f'dbt-{dbt_version}',
-                secure=credentials.secure,
-                verify=credentials.verify,
+                interface='https' if credentials.secure else 'http',
+                compress=False if credentials.compression == '' else bool(credentials.compression),
                 connect_timeout=credentials.connect_timeout,
                 send_receive_timeout=credentials.send_receive_timeout,
-                sync_request_timeout=credentials.sync_request_timeout,
-                compress_block_size=credentials.compress_block_size,
-                compression=False if credentials.compression == '' else credentials.compression,
-                **kwargs,
+                verify=credentials.verify,
+                query_limit=0,
+                session_id='dbt::' + str(uuid.uuid4()),
+                **custom_settings,
             )
             connection.handle = handle
             connection.state = 'open'
-        except errors.ServerException as e:
+        except DatabaseError as err:
             logger.debug(
                 'Got an error when attempting to open a clickhouse connection: \'{}\'',
-                str(e),
+                str(err),
             )
 
             connection.handle = None
             connection.state = 'fail'
 
-            raise dbt.exceptions.FailedToConnectException(str(e))
+            raise dbt.exceptions.FailedToConnectException(str(err))
 
         return connection
 
@@ -135,9 +147,12 @@ class ClickhouseConnectionManager(SQLConnectionManager):
         logger.debug('Cancel query \'{}\'', connection_name)
 
     @classmethod
-    def get_table_from_response(cls, response, columns) -> agate.Table:
-        column_names = [x[0] for x in columns]
-
+    def get_table_from_response(cls, response, column_names) -> agate.Table:
+        """
+        Build agate tabel from response.
+        :param response: ClickHouse query result
+        :param column_names: Table column names
+        """
         data = []
         for row in response:
             data.append(dict(zip(column_names, row)))
@@ -152,24 +167,23 @@ class ClickhouseConnectionManager(SQLConnectionManager):
         client = conn.handle
 
         with self.exception_handler(sql):
-            logger.debug(
-                'On {connection_name}: {sql}'.format(connection_name=conn.name, sql=f'{sql}...'),
-            )
+            logger.debug(f'On {conn.name}: {sql}...')
 
             pre = time.time()
 
-            response, columns = client.execute(sql, with_column_types=True)
+            if fetch:
+                query_result = client.query(sql)
+            else:
+                query_result = client.command(sql)
 
             status = self.get_status(client)
 
-            logger.debug(
-                'SQL status: {status} in {elapsed:0.2f} seconds'.format(
-                    status=status, elapsed=(time.time() - pre)
-                ),
-            )
+            logger.debug(f'SQL status: {status} in {(time.time() - pre):.2f} seconds')
 
             if fetch:
-                table = self.get_table_from_response(response, columns)
+                table = self.get_table_from_response(
+                    query_result.result_set, query_result.column_names
+                )
             else:
                 table = dbt.clients.agate_helper.empty_table()
             return status, table
@@ -186,33 +200,33 @@ class ClickhouseConnectionManager(SQLConnectionManager):
         client = conn.handle
 
         with self.exception_handler(sql):
-            logger.debug(
-                'On {connection_name}: {sql}'.format(connection_name=conn.name, sql=f'{sql}...')
-            )
+            logger.debug(f'On {conn.name}: {sql}...')
 
             pre = time.time()
-            client.execute(sql)
+            client.command(sql)
 
             status = self.get_status(client)
 
-            logger.debug(
-                'SQL status: {status} in {elapsed:0.2f} seconds'.format(
-                    status=status, elapsed=(time.time() - pre)
-                )
-            )
+            logger.debug(f'SQL status: {status} in {(time.time() - pre):0.2f} seconds')
 
             return conn, None
 
     @classmethod
     def get_credentials(cls, credentials):
+        """
+        Returns ClickHouse credentials
+        """
         return credentials
 
     @classmethod
-    def get_status(cls, cursor):
+    def get_status(cls, _):
+        """
+        Returns connection status
+        """
         return 'OK'
 
     @classmethod
-    def get_response(cls, cursor):
+    def get_response(cls, _):
         return 'OK'
 
     def begin(self):
