@@ -13,6 +13,7 @@ from dbt.adapters.sql import SQLAdapter
 from dbt.clients.agate_helper import table_from_rows
 from dbt.contracts.graph.manifest import Manifest
 from dbt.contracts.relation import RelationType
+from dbt.events import AdapterLogger
 from dbt.utils import executor
 
 from dbt.adapters.clickhouse.column import ClickhouseColumn
@@ -36,6 +37,7 @@ class ClickhouseAdapter(SQLAdapter):
     Column = ClickhouseColumn
     ConnectionManager = ClickhouseConnectionManager
     AdapterSpecificConfigs = ClickhouseConfig
+    logger = AdapterLogger("dbt_clickhouse_tests")
 
     @classmethod
     def date_function(cls):
@@ -52,7 +54,7 @@ class ClickhouseAdapter(SQLAdapter):
 
     @classmethod
     def convert_boolean_type(cls, agate_table: agate.Table, col_idx: int) -> str:
-        return 'UInt8'
+        return 'Bool'
 
     @classmethod
     def convert_datetime_type(cls, agate_table: agate.Table, col_idx: int) -> str:
@@ -105,7 +107,7 @@ class ClickhouseAdapter(SQLAdapter):
 
         return relations
 
-    def get_relation(self, database: str, schema: str, identifier: str):
+    def get_relation(self, database: Optional[str], schema: str, identifier: str):
         if not self.Relation.include_policy.database:
             database = None
 
@@ -226,13 +228,39 @@ class ClickhouseAdapter(SQLAdapter):
         csv_funcs = [c.csvify for c in table._column_types]
 
         buf = io.StringIO()
-        writer = csv.writer(buf)
+        writer = csv.writer(buf, lineterminator="\n")
 
         for row in table.rows:
             writer.writerow(tuple(csv_funcs[i](d) for i, d in enumerate(row)))
 
         return buf.getvalue()
 
+    def run_sql_for_tests(self, sql, fetch, conn):
+        client = conn.handle
+        try:
+            if fetch:
+                result = client.query(sql).result_set
+            else:
+                result = client.command(sql)
+            if fetch == "one" and len(result) > 0:
+                return result[0]
+            if fetch == "all":
+                return result
+        except BaseException as e:
+            self.logger.error(sql)
+            self.logger.error(e)
+            raise
+        finally:
+            conn.state = 'close'
+
+    @available
+    def get_model_settings(self, model):
+        settings = model['config'].get('settings', dict())
+        res = []
+        for key in settings:
+            res.append(f' {key}={settings[key]}')
+        return '' if len(res) == 0 else 'SETTINGS ' + ', '.join(res) + '\n'
+
 
 def _expect_row_value(key: str, row: agate.Row):
     if key not in row.keys():
